@@ -33,7 +33,7 @@ from transformers import AdamW, get_linear_schedule_with_warmup
 from torch.utils.data import DataLoader, TensorDataset
 from torch.utils.data import RandomSampler, SequentialSampler
 from torch.utils.data import random_split
-
+from collections import OrderedDict
 
 warnings.filterwarnings("ignore")
 
@@ -61,7 +61,6 @@ class BertTagger:
         self.config = Config()
         self.test_mode = test_mode
         self.use_gpu = use_gpu
-        #self.cache_dir = cache_dir
 
         self.device_available_memory = None
         self.device_type = ""
@@ -231,7 +230,7 @@ class BertTagger:
         return bert_model_name
 
 
-    def load_pretrained(self, state_dict=None):
+    def load_pretrained(self, state_dict = OrderedDict()):
         """ Load pretrained BERT model.
 
         Parameters:
@@ -246,13 +245,16 @@ class BertTagger:
             model_to_load = model_dir
             model_downloaded = True
 
+        if self.config.use_state_dict:
+            # Initializing state_dict as None forces the usage of default state dict (not an empty one)
+            state_dict = None
+
         model_kwargs = {"state_dict": state_dict, "num_labels": self.config.n_classes}
 
         if self.cache_dir:
              model_kwargs.update({"cache_dir": self.cache_dir})
 
         if not BertTagger.is_roberta_model(model_to_load):
-            #os.environ['TRANSFORMERS_CACHE'] = self.cache_dir
 
             model = BertForSequenceClassification.from_pretrained(model_to_load, **model_kwargs)
         else:
@@ -276,14 +278,17 @@ class BertTagger:
 
 
     @staticmethod
-    def save_pretrained_model(bert_model: str, save_dir: str, state_dict: json = None, logger = None):
+    def save_pretrained_model(bert_model: str, save_dir: str, state_dict: OrderedDict = None, logger: logging.Logger = None, cache_dir: str = "", num_labels: int = None):
         model_dir = os.path.join(save_dir, BertTagger.normalize_name(bert_model), "model")
         if not os.path.exists(model_dir) or len(os.listdir(model_dir)) < 1:
             if logger: logger.info(f"Downloading pretrained model {bert_model}...")
 
             model_kwargs = {"state_dict": state_dict}
-            if self.cache_dir:
-                 model_kwargs.update({"cache_dir": self.cache_dir})
+
+            if num_labels:
+                model_kwargs.update({"num_labels": num_labels})
+            if cache_dir:
+                 model_kwargs.update({"cache_dir": cache_dir})
 
             if not BertTagger.is_roberta_model(bert_model):
                 model = BertForSequenceClassification.from_pretrained(bert_model, **model_kwargs)
@@ -296,13 +301,13 @@ class BertTagger:
 
 
     @staticmethod
-    def save_pretrained_tokenizer(bert_model: str, save_dir: str, logger = None):
+    def save_pretrained_tokenizer(bert_model: str, save_dir: str, logger: logging.Logger = None, cache_dir: str = ""):
         tokenizer_dir = os.path.join(save_dir, BertTagger.normalize_name(bert_model), "tokenizer")
         if not os.path.exists(tokenizer_dir) or len(os.listdir(tokenizer_dir)) < 1:
             if logger: logger.info(f"Downloading pretrained tokenizer {bert_model}...")
             model_kwargs = {}
-            if self.cache_dir:
-                 model_kwargs.update({"cache_dir": self.cache_dir})
+            if cache_dir:
+                 model_kwargs.update({"cache_dir": cache_dir})
 
             if not BertTagger.is_roberta_model(bert_model):
                 tokenizer = BertTokenizer.from_pretrained(bert_model, **model_kwargs)
@@ -315,19 +320,20 @@ class BertTagger:
 
 
     @staticmethod
-    def download_pretrained_models(bert_models: List[str], save_dir: str, logger = None):
+    def download_pretrained_models(bert_models: List[str], save_dir: str, logger: logging.Logger = None, cache_dir: str = "", num_labels: int = None):
         """ Download pretrained BERT models and tokenizers.
 
         Parameters:
             bert_models - Names of the BERT models to download.
             save_dir - Directory where to save the pretrained models and tokenizers.
+            cache_dir - Directory where to save cache files.
         """
         failed_models = []
         errors = []
         for bert_model in bert_models:
             try:
-                BertTagger.save_pretrained_tokenizer(bert_model, save_dir, logger=logger)
-                BertTagger.save_pretrained_model(bert_model, save_dir, logger=logger)
+                BertTagger.save_pretrained_tokenizer(bert_model, save_dir, logger=logger, cache_dir = cache_dir)
+                BertTagger.save_pretrained_model(bert_model, save_dir, logger=logger, cache_dir = cache_dir, num_labels = num_labels)
             except:
                 failed_models.append(bert_model)
         if failed_models:
